@@ -48,11 +48,13 @@ exists, join, relpath = os.path.exists, os.path.join, os.path.relpath
 OEB_FONTS = {guess_type('a.ttf'), guess_type('b.otf'), guess_type('a.woff'), 'application/x-font-ttf', 'application/x-font-otf', 'application/font-sfnt'}
 OPF_NAMESPACES = {'opf':OPF2_NS, 'dc':DC11_NS}
 
+
 class CSSPreProcessor(cssp):
 
     def __call__(self, data):
         return self.MS_PAT.sub(self.ms_sub, data)
 
+
 def clone_dir(src, dest):
     ' Clone a directory using hard links for the files, dest must already exist '
     for x in os.listdir(src):
@@ -67,6 +69,7 @@ def clone_dir(src, dest):
             except:
                 shutil.copy2(spath, dpath)
 
+
 def clone_container(container, dest_dir):
     ' Efficiently clone a container using hard links '
     dest_dir = os.path.abspath(os.path.realpath(dest_dir))
@@ -76,32 +79,39 @@ def clone_container(container, dest_dir):
         return cls(None, None, container.log, clone_data=clone_data)
     return cls(None, container.log, clone_data=clone_data)
 
+
 def name_to_abspath(name, root):
     return os.path.abspath(join(root, *name.split('/')))
 
+
 def abspath_to_name(path, root):
     return relpath(os.path.abspath(path), root).replace(os.sep, '/')
 
+
 def name_to_href(name, root, base=None, quote=urlquote):
     fullpath = name_to_abspath(name, root)
     basepath = root if base is None else os.path.dirname(name_to_abspath(base, root))
     path = relpath(fullpath, basepath).replace(os.sep, '/')
     return quote(path)
 
+
 def href_to_name(href, root, base=None):
     base = root if base is None else os.path.dirname(name_to_abspath(base, root))
-    purl = urlparse(href)
+    try:
+        purl = urlparse(href)
+    except ValueError:
+        return None
     if purl.scheme or not purl.path:
         return None
     href = urlunquote(purl.path)
-    if href.startswith('/') or (len(href) > 1 and href[1] == ':' and 'a' <= href[0].lower() <= 'z'):
-        # For paths that start with drive letter os.path.join(base, href)
-        # will discard base and return href on windows, so we assume that
-        # such paths are also absolute paths, on all platforms.
+    if iswindows and ':' in href:
+        # path manipulations on windows fail for paths with : in them, so we
+        # assume all such paths are invalid/absolute paths.
         return None
     fullpath = os.path.join(base, *href.split('/'))
     return abspath_to_name(fullpath, root)
 
+
 class ContainerBase(object):  # {{{
     '''
     A base class that implements just the parsing methods. Useful to create
@@ -190,6 +200,7 @@ class ContainerBase(object):  # {{{
                          css_preprocessor=(None if self.tweak_mode else self.css_preprocessor))
 # }}}
 
+
 class Container(ContainerBase):  # {{{
 
     '''
@@ -335,7 +346,10 @@ class Container(ContainerBase):  # {{{
         if not os.path.exists(base):
             os.makedirs(base)
         with lopen(path, 'wb') as f:
-            f.write(data)
+            if hasattr(data, 'read'):
+                shutil.copyfileobj(data, f)
+            else:
+                f.write(data)
         mt = media_type or self.guess_type(name)
         self.name_path_map[name] = path
         self.mime_map[name] = mt
@@ -469,7 +483,7 @@ class Container(ContainerBase):  # {{{
         return name_to_abspath(name, self.root)
 
     def exists(self, name):
-        ''' True iff a file corresponding to the canonical name exists. Note
+        ''' True iff a file/directory corresponding to the canonical name exists. Note
         that this function suffers from the limitations of the underlying OS
         filesystem, in particular case (in)sensitivity. So on a case
         insensitive filesystem this will return True even if the case of name
@@ -1004,9 +1018,12 @@ class Container(ContainerBase):  # {{{
 # }}}
 
 # EPUB {{{
+
+
 class InvalidEpub(InvalidBook):
     pass
 
+
 class ObfuscationKeyMissing(InvalidEpub):
     pass
 
@@ -1014,6 +1031,7 @@ OCF_NS = 'urn:oasis:names:tc:opendocument:xmlns:container'
 VCS_IGNORE_FILES = frozenset('.gitignore .hgignore .agignore .bzrignore'.split())
 VCS_DIRS = frozenset(('.git', '.hg', '.svn', '.bzr'))
 
+
 def walk_dir(basedir):
     for dirpath, dirnames, filenames in os.walk(basedir):
         for vcsdir in VCS_DIRS:
@@ -1027,6 +1045,7 @@ def walk_dir(basedir):
             if fname not in VCS_IGNORE_FILES:
                 yield is_root, dirpath, fname
 
+
 class EpubContainer(Container):
 
     book_type = 'epub'
@@ -1195,7 +1214,7 @@ class EpubContainer(Container):
                     break
         if raw_unique_identifier is not None:
             idpf_key = raw_unique_identifier
-            idpf_key = re.sub(u'\u0020\u0009\u000d\u000a', u'', idpf_key)
+            idpf_key = re.sub(u'[\u0020\u0009\u000d\u000a]', u'', idpf_key)
             idpf_key = hashlib.sha1(idpf_key.encode('utf-8')).digest()
         key = None
         for item in self.opf_xpath('//*[local-name()="metadata"]/*'
@@ -1277,6 +1296,7 @@ class EpubContainer(Container):
     def path_to_ebook(self):
         def fget(self):
             return self.pathtoepub
+
         def fset(self, val):
             self.pathtoepub = val
         return property(fget=fget, fset=fset)
@@ -1284,9 +1304,12 @@ class EpubContainer(Container):
 # }}}
 
 # AZW3 {{{
+
+
 class InvalidMobi(InvalidBook):
     pass
 
+
 def do_explode(path, dest):
     from calibre.ebooks.mobi.reader.mobi6 import MobiReader
     from calibre.ebooks.mobi.reader.mobi8 import Mobi8Reader
@@ -1323,11 +1346,13 @@ def opf_to_azw3(opf, outpath, container):
     set_cover(oeb)
     outp.convert(oeb, outpath, inp, plumber.opts, container.log)
 
+
 def epub_to_azw3(epub, outpath=None):
     container = get_container(epub, tweak_mode=True)
     outpath = outpath or (epub.rpartition('.')[0] + '.azw3')
     opf_to_azw3(container.name_to_abspath(container.opf_name), outpath, container)
 
+
 class AZW3Container(Container):
 
     book_type = 'azw3'
@@ -1398,6 +1423,7 @@ class AZW3Container(Container):
     def path_to_ebook(self):
         def fget(self):
             return self.pathtoazw3
+
         def fset(self, val):
             self.pathtoazw3 = val
         return property(fget=fget, fset=fset)
@@ -1407,6 +1433,7 @@ class AZW3Container(Container):
         return set(self.name_path_map)
 # }}}
 
+
 def get_container(path, log=None, tdir=None, tweak_mode=False):
     if log is None:
         log = default_log
@@ -1419,6 +1446,7 @@ def get_container(path, log=None, tdir=None, tweak_mode=False):
     ebook.tweak_mode = tweak_mode
     return ebook
 
+
 def test_roundtrip():
     ebook = get_container(sys.argv[-1])
     p = PersistentTemporaryFile(suffix='.'+sys.argv[-1].rpartition('.')[-1])
@@ -1432,5 +1460,3 @@ def test_roundtrip():
 
 if __name__ == '__main__':
     test_roundtrip()
-
-
